@@ -17,8 +17,13 @@
  * USA
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
-#include "gnome.h"
+#endif
+
+#include <string.h>
+#include <gnome.h>
+
 #include "sol.h"
 #include "menu.h"
 #include "dialog.h"
@@ -113,7 +118,7 @@ void help_about_callback ()
 			      "played."),
 			    (const char **)authors,
 			    (const char **)documenters,
-			    strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+			    g_str_equal (translator_credits, "translator_credits") ? NULL : translator_credits,
 			    pixbuf);
 	
 	if (pixbuf != NULL)
@@ -170,7 +175,7 @@ GnomeUIInfo file_menu[] = {
   GNOMEUIINFO_END
 };
 
-GnomeUIInfo settings_menu[] = {
+GnomeUIInfo view_menu[] = {
 
 #if 0
   GNOMEUIINFO_ITEM_STOCK(N_("Game _options..."),
@@ -183,7 +188,8 @@ GnomeUIInfo settings_menu[] = {
   GNOMEUIINFO_TOGGLEITEM (N_("_Toolbar"), N_("Show or hide the toolbar"),
                           toolbar_toggle_callback, NULL),
 
-  GNOMEUIINFO_MENU_PREFERENCES_ITEM(show_preferences_dialog, NULL),
+  GNOMEUIINFO_ITEM_STOCK (N_("_Cards..."), N_("Change the appearance of the cards"), 
+			  show_preferences_dialog, NULL),
 
   GNOMEUIINFO_END
 };
@@ -232,7 +238,7 @@ GnomeUIInfo top_menu[] = {
 
   GNOMEUIINFO_MENU_GAME_TREE(game_menu),
 
-  GNOMEUIINFO_MENU_SETTINGS_TREE(settings_menu),
+  GNOMEUIINFO_MENU_VIEW_TREE(view_menu),
 
   GNOMEUIINFO_MENU_HELP_TREE(help_menu),
 
@@ -281,7 +287,7 @@ void create_menus ()
   gnome_app_create_menus (GNOME_APP(app), top_menu);
   gnome_app_create_toolbar (GNOME_APP(app), toolbar);
 
-  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (settings_menu[0].widget),
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (view_menu[0].widget),
                                   gconf_client_get_bool (gconf_client,
                                                          "/apps/aisleriot/show_toolbar",
                                                          NULL));
