@@ -1,3 +1,27 @@
+/* -*- mode:C; tab-width:8; c-basic-offset:8; indent-tabs-mode:true -*- */
+
+/*
+ * Mines for GNOME
+ * Author:        Pista <szekeres@cyberspace.mht.bme.hu>
+ *
+ * Score support: horape@compendium.com.ar
+ * Mine Resizing: djb@redhat.com
+ *
+ * This game is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * USA
+ */
 
 #include <time.h>
 #include <gtk/gtk.h>
@@ -85,18 +109,22 @@ static inline gint cell_idx(GtkMineField *mfield, guint x, guint y)
 
 static void _setup_sign (sign *signp, const char *file, guint minesize)
 {
-	GdkPixbuf *image, *scaledimage;;
+	GdkPixbuf *scaledimage;
 
-	/* TODO: catch GError */
-	image = gdk_pixbuf_new_from_file (file, NULL);
-        scaledimage = gdk_pixbuf_scale_simple (image, minesize - 2, minesize - 2,
+	if (signp->pixbuf == NULL) {
+		GError *error = NULL;
+		signp->pixbuf = gdk_pixbuf_new_from_file (file, &error);
+		if (signp->pixbuf == NULL)
+			g_error (error->message);
+	}	
+
+        scaledimage = gdk_pixbuf_scale_simple (signp->pixbuf, minesize - 2, minesize - 2,
                                                GDK_INTERP_BILINEAR);
         
 	gdk_pixbuf_render_pixmap_and_mask (scaledimage, &signp->pixmap,
 					   &signp->mask, 127);
 
         g_object_unref (scaledimage);
-	g_object_unref (image);
 	gdk_drawable_get_size(signp->pixmap, &(signp->width), &(signp->height));
 }
 
@@ -226,11 +254,12 @@ static void gtk_minefield_unrealize (GtkWidget *widget)
 static void gtk_minefield_size_allocate (GtkWidget     *widget,
 				 	 GtkAllocation *allocation)
 {
-        widget->allocation = *allocation;
         guint minesize, width, height;
         guint xofs, yofs;
         GtkMineField * mfield;
 
+        widget->allocation = *allocation;
+
         mfield = GTK_MINEFIELD (widget);
         
 	if (GTK_WIDGET_REALIZED (widget)) {
@@ -260,13 +289,15 @@ static void gtk_mine_draw(GtkMineField *mfield, guint x, guint y)
 {
         int c = cell_idx(mfield, x, y);
 	int noshadow;
+	gboolean clicked;
 	int n;
 	guint minesize;
         static GdkGC *dots;
         static char stipple_data[]  = { 0x03, 0x03, 0x0c, 0x0c };
         static GdkPixmap *stipple = NULL;
         GtkWidget *widget = GTK_WIDGET(mfield);
-
+	GdkRectangle rect;
+	
         /* This gives us a dotted line to increase the contrast between
          * buttons and the "sea". */
         if (stipple == NULL) {
@@ -279,17 +310,31 @@ static void gtk_mine_draw(GtkMineField *mfield, guint x, guint y)
         }
                 
 	minesize = mfield->minesize;
-
+	
 	if (mfield->lose || mfield->win) {
 		noshadow = mfield->mines[c].shown;
 	} else {
-		noshadow = mfield->mines[c].down || mfield->mines[c].shown;
+		noshadow = mfield->mines[c].shown;
 	}
-	
-	gdk_window_clear_area(widget->window, x * minesize, y * minesize,
-	                      minesize, minesize);
+
+	clicked = mfield->mines[c].down;
+
+	/* gtk_paint_box needs a clipping rectangle. */
+	rect.x = x*minesize;
+	rect.y = y*minesize;
+	rect.width = minesize;
+	rect.height = minesize;
 
 	if (noshadow) { /* draw grid on ocean floor */
+		gtk_paint_box (widget->style,
+				    widget->window,
+				    GTK_STATE_NORMAL,
+				    clicked ? GTK_SHADOW_IN : GTK_SHADOW_NONE,
+				    &rect,
+				    widget,
+				    "button",
+				    x*minesize, y*minesize,
+				    minesize, minesize);
 		if (y == 0) {	/* top row only */
 			gdk_draw_line(widget->window,	/* top */
 			              dots,
@@ -320,10 +365,15 @@ static void gtk_mine_draw(GtkMineField *mfield, guint x, guint y)
                               y*minesize+minesize-1);
 
 	} else {	/* draw shadow around possible mine location */
-		gtk_paint_shadow(widget->style, widget->window,
-		                GTK_WIDGET_STATE (widget), GTK_SHADOW_OUT,
-				NULL, widget, NULL,
-		                x*minesize, y*minesize, minesize, minesize);
+		gtk_paint_box (widget->style,
+			       widget->window,
+			       GTK_STATE_SELECTED,
+			       clicked ? GTK_SHADOW_IN : GTK_SHADOW_OUT,
+			       &rect,
+			       widget,
+			       "button",
+			       x*minesize, y*minesize,
+			       minesize, minesize);
         }
 
 	if (mfield->mines[c].shown && !mfield->mines[c].mined) {
@@ -565,14 +615,10 @@ static void gtk_minefield_randomize (GtkMineField *mfield, int curloc)
 	guint x, y;
 	guint n;
 	guint cidx;
-	static GRand *grand = NULL;
-
-	if (grand == NULL)
-		grand = g_rand_new ();
 
 	/* randomly set the mines, but avoid the current location (why ?)*/
 	for (n = 0; n < mfield->mcount; ) {
-	        i = g_rand_int_range (grand, 0, mfield->xsize * mfield->ysize);
+	        i = g_rand_int_range (mfield->grand, 0, mfield->xsize * mfield->ysize);
 		if (!mfield->mines[i].mined && i != curloc) {
 		        mfield->mines[i].mined = 1;
 			n++;
@@ -631,6 +677,15 @@ static void gtk_minefield_toggle_mark(GtkMineField *mfield, guint x, guint y)
 
 	switch (mfield->mines[c].marked) {
 	case MINE_NOMARK:
+		/* If we've used all the flags don't plant any more,
+		 * this should be an indication to the player that they
+		 * have made a mistake. */
+		if (mfield->flag_count == mfield->mcount) {
+			if (mfield->use_question_marks) {
+				mfield->mines[c].marked = MINE_QUESTION;
+			}
+			break;
+		}
 		mfield->mines[c].marked = MINE_MARKED;
 		mfield->flag_count++;
 		break;
@@ -653,9 +708,6 @@ static void gtk_minefield_toggle_mark(GtkMineField *mfield, guint x, guint y)
 		g_signal_emit(GTK_OBJECT(mfield),
 				minefield_signals[MARKS_CHANGED_SIGNAL],
 				0, NULL);
-	if (mfield->shown == mfield->xsize * mfield->ysize-mfield->mcount) {
-                        gtk_minefield_win(mfield);
-		}
 }
 
 static inline void gtk_minefield_multi_press(GtkMineField *mfield,
@@ -941,6 +993,11 @@ static void gtk_minefield_init (GtkMineField *mfield)
 	mfield->mines = NULL;
 	mfield->started = FALSE;
 	mfield->cdown = -1;
+
+	mfield->flag.pixbuf = NULL;
+        mfield->mine.pixbuf = NULL;
+	mfield->question.pixbuf = NULL;
+	mfield->grand = g_rand_new ();
 }
 
 void gtk_minefield_set_size(GtkMineField *mfield, guint xsize, guint ysize)
@@ -1032,3 +1089,95 @@ void gtk_minefield_set_use_question_marks(GtkMineField *mfield, gboolean use_que
 	
 	mfield->use_question_marks = use_question_marks;
 }
+
+/* Hunt for a hint to give the player. Revealed squares are handled here,
+ * everything else is passed back up. The comments below detail the
+ * strategy for revealing squares. */
+gint gtk_minefield_hint (GtkMineField *mfield)
+{
+	gint i,x,y;
+	gint a,c;
+	mine * m;
+	guint ncase1, ncase2, ncase3;
+	guint *case1list, *case2list, *case3list;
+	guint *case1ptr, *case2ptr, *case3ptr;
+	gint size;
+	
+	g_return_val_if_fail (mfield != NULL, MINEFIELD_HINT_NO_GAME);
+	g_return_val_if_fail (GTK_IS_MINEFIELD(mfield), MINEFIELD_HINT_NO_GAME);
+	if (!mfield->in_play)
+		return MINEFIELD_HINT_NO_GAME;
+	
+	/* We search for three cases:
+	 *
+	 * Case 1: we look for squares adjacent to both a mine and a revealed
+	 * square since these are most likely to help the player and resolve
+	 * ambiguous situations.
+	 *
+	 * Case 2: we look for squares that are adjacent to a mine
+	 * (this will only occur in the rare case that a square is completely
+	 * encircled by mines, but at that point this case is probably
+	 * useful).
+	 *
+	 * Case 3: we look for any unrevealed square without a mine (as a
+	 * consequence of the previous cases this won't be adjacent to a
+	 * mine).
+	 */
+
+	/* This code is pretty diabolical,
+	 * Yet it is perfectly logical,
+	 * Is it C ?
+	 * Is it me ?
+	 * Or is it just pathological ?
+	 */
+	
+	size = mfield->xsize*mfield->ysize;
+	case1ptr = case1list = g_malloc (size*sizeof(guint));
+	case2ptr = case2list = g_malloc (size*sizeof(guint));
+	case3ptr = case3list = g_malloc (size*sizeof(guint));
+	ncase1 = ncase2 = ncase3 = 0;
+	g_return_val_if_fail (case1ptr && case2ptr && case3ptr,
+			      MINEFIELD_HINT_NO_GAME);
+	
+	for (i = 0; i<size; i++) {
+		m = mfield->mines + i;
+		if (!m->mined && !m->marked && !m->shown) {
+			ncase3++;
+			*case3ptr++ = i;
+			if (m->neighbours > 0) {
+				ncase2++;
+				*case2ptr++ = i;
+				for (a = 0; a < 8; a++) {
+					x = i % mfield->xsize;
+					y = i / mfield->ysize;
+					c = cell_idx (mfield,
+						      x + neighbour_map[a].x,
+						      y + neighbour_map[a].y);
+					if ((c != -1) &&
+					    mfield->mines[c].shown) {
+						ncase1++;
+						*case1ptr++ = i;
+					}
+				}
+			}
+		}
+	}
+
+	if (ncase1 > 0) {
+		a = g_rand_int_range (mfield->grand, 0, ncase1);
+		i = case1list[a];
+	} else if (ncase2 > 0) {
+		a = g_rand_int_range (mfield->grand, 0, ncase2);
+		i = case2list[a];
+	} else if (ncase3 > 0) {
+		a = g_rand_int_range (mfield->grand, 0, ncase3);
+		i = case3list[a];
+	} else return MINEFIELD_HINT_ALL_MINES;
+
+	x = i % mfield->xsize;
+	y = i / mfield->ysize;
+
+	gtk_minefield_show (mfield, x, y);
+	gtk_mine_draw (mfield, x, y);
+	return MINEFIELD_HINT_ACCEPTED;
+}
